# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Policy Rule extension unit tests
"""
#from networking_plumgrid.neutron.plugins.common import \
#    exceptions as p_excep
#from networking_plumgrid.neutron.plugins.common import \
#    policy_exceptions as policy_excep
from networking_plumgrid.neutron.plugins.extensions import \
    policyrule as ext_pr
from networking_plumgrid.neutron.tests.unit import \
    test_networking_plumgrid as test_pg

from neutron.api import extensions
#from neutron import context
#from neutron import manager
from neutron.tests.unit.api import test_extensions as test_ext
from oslo_log import log as logging
#import uuid

LOG = logging.getLogger(__name__)


class PolicyRuleExtensionManager(object):

    def get_resources(self):
        return ext_pr.Policyrule.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class PolicyRuleTestCase(test_pg.PLUMgridPluginV2TestCase):
    def setUp(self, plugin=None, ext_mgr=None):
        super(PolicyRuleTestCase, self).setUp()
        ext_mgr = PolicyRuleExtensionManager()
        extensions.PluginAwareExtensionManager._instance = None
        self.ext_api = test_ext.setup_extensions_middleware(ext_mgr)


class TestPolicyRule(PolicyRuleTestCase):

    def test_create_policy_rule(self):
        pass

    def test_create_policy_rule_src_grp_epg(self):
        pass

    def test_create_policy_rule_dst_grp_epg(self):
        pass

    def test_create_policy_rule_src_grp_epg_invalid(self):
        pass

    def test_create_policy_rule_dst_grp_epg_invalid(self):
        pass

    def test_create_policy_rule_src_dst_grp_epg(self):
        pass

    def test_create_policy_rule_src_grp_sg(self):
        pass

    def test_create_policy_rule_dst_grp_sg(self):
        pass

    def test_create_policy_rule_src_grp_sg_invalid(self):
        pass

    def test_create_policy_rule_dst_grp_sg_invalid(self):
        pass

    def test_create_policy_rule_src_dst_grp_sg(self):
        pass

    def test_create_policy_rule_src_epg_dst_sg(self):
        pass

    def test_create_policy_rule_src_sg_dst_epg(self):
        pass

    def test_create_policy_rule_action_allow(self):
        pass

    def test_create_policy_rule_action_copy(self):
        pass

    def test_create_policy_rule_action_invalid(self):
        pass

    def test_create_policy_rule_protocol_any(self):
        pass

    def test_create_policy_rule_protocol_tcp(self):
        pass

    def test_create_policy_rule_protocol_udp(self):
        pass

    def test_create_policy_rule_protocol_invalid(self):
        pass

    def test_create_policy_rule_icmp_with_range(self):
        pass

    def test_create_policy_rule_src_port_range(self):
        pass

    def test_create_policy_rule_dst_port_range(self):
        pass

    def test_create_policy_rule_invalid_src_port_range(self):
        pass

    def test_create_policy_rule_invalid_dst_port_range(self):
        pass

    def test_create_policy_rule_action_target(self):
        pass

    def test_create_policy_rule_remote_action_target(self):
        pass

    def test_create_policy_rule_src_grp_tag(self):
        pass

    def test_create_policy_rule_dst_grp_tag(self):
        pass

    def test_create_policy_rule_global_tag(self):
        pass

    def test_create_policy_rule_src_grp_tag_not_associated_with_src_grp(self):
        pass

    def test_create_policy_rule_dst_grp_tag_not_associated_with_dst_grp(self):
        pass

    def test_create_policy_rule_global_tag_does_not_exist(self):
        pass

    def test_create_policy_rule_action_target_doest_not_exist(self):
        pass

    def test_create_policy_rule_remote_action_target_does_not_exist(self):
        pass

    def test_delete_policy_rule(self):
        pass

    def test_delete_policy_rule_does_not_exist(self):
        pass

    def test_list_policy_rules(self):
        pass

    def test_create_get_policy_rule(self):
        pass

    def _make_epg_dict(self, ptag_id=None,
                       name="test_sg_name"):
        epg_dict = {"endpoint_group": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "description": "test_description",
                   "is_security_group": False}}
        if ptag_id is not None:
            epg_dict["endpoint_group"]["policy_tag_id"] = ptag_id
        return epg_dict

    def _make_epg_update_dict(self, name="test_tenant",
                              description="test_description",
                              add_tag=[],
                              remove_tag=[]):
        return {"endpoint_group": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "description": description,
                   "add_tag": add_tag,
                   "remove_tag": remove_tag,
                   "is_security_group": False,
                   "policy_tag_id": None}}

    def _make_ps_dict(self, name="test_ps", inports=[], eports=[],
                      bidirect_ports=[]):
        return {"policy_service": {
                    "tenant_id": "test_tenant",
                    "description": "test-ps",
                    "name": name,
                    "ingress_ports": inports,
                    "egress_ports": eports,
                    "bidirectional_ports": bidirect_ports}}

    def _make_pr_dict(self, action_target=None, name="test_policy",
                      src_grp=None, dst_grp=None, action="copy",
                      protocol="any", src_port_range=None,
                      dst_port_range=None, tag=None):
        return {"endpoint_policy": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "src_grp": src_grp,
                   "dst_grp": dst_grp,
                   "action": action,
                   "src_port_range": src_port_range,
                   "dst_port_range": dst_port_range,
                   "protocol": protocol,
                   "action_target": action_target,
                   "tag": tag}}

    def _make_ptag_dict(self, tag_type, tag_id=None,
                        floatingip_id=None, floating_ip_address=None,
                        router_id=None):
        return {"policy_tag": {
                               "tenant_id": "test_tenant",
                               "name": "test_name",
                               "tag_type": tag_type,
                               "tag_id": tag_id,
                               "floatingip_id": floatingip_id,
                               "floating_ip_address": floating_ip_address,
                               "router_id": router_id}}

    def _fake_sg(self):
        return {"security_group": {"name": "fake-sg",
                                   "description": "sample-description",
                                   "tenant_id": "test-tenant"}}



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from tempest import clients
from tempest import config
from tempest import test

from plumgrid_tempest_plugin.services.pap import pap_client
from plumgrid_tempest_plugin.services.pap import td_client

CONF = config.CONF


class Manager(clients.Manager):
    def __init__(self, credentials=None, service=None):
        super(Manager, self).__init__(credentials, service)
        params = {
            'service': 'network',
            'region': CONF.identity.region,
            'endpoint_type': CONF.network.endpoint_type,
            'build_interval': CONF.network.build_interval,
            'build_timeout': CONF.network.build_timeout
        }
        params.update(self.default_params)
        self.td_client = td_client.TDClient(self.auth_provider, **params)
        self.pap_client = pap_client.PAPClient(self.auth_provider, **params)


class BaseTest(test.BaseTestCase):
    """Base class for PLUMgrid tests."""

    # Use the PLUMgrid Client Manager
    client_manager = Manager

    # NOTE: credentials holds a list of the credentials to be allocated
    # at class setup time. Credential types can be 'primary', 'alt', 'admin' or
    # a list of roles - the first element of the list being a label, and the
    # rest the actual roles.
    # NOTE: primary will result in a manager @ cls.os, alt will have
    # cls.os_alt, and admin will have cls.os_adm.
    credentials = ['primary']



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from tempest.lib.common.utils import data_utils

from plumgrid_tempest_plugin.services import base


PG_PAP = 'physical-attachment-points'


class PAPClient(base.ClientBase):
    """API Tempest REST client for Physical Attachment Point API"""

    # Set appropriate API version
    uri_prefix = 'v2.0'

    def create_pap(self, name=None, lacp=False, hash_mode='L2',
                   interfaces=[], params=None):
        """Create a PAP with the specified parameters.

        :param name: The name of the Physical Attachment Point.
            Default: Random Value
        :param lacp: enable/disable LACP on Physical Attachment Point.
            Default: False
        :param hash_mode: hash_mode config for Physical Attachment Point.
            Default: L2
        :param interfaces: List of gateway interfaces.
            Default: None
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the created pap.
        """
        pap = {"physical_attachment_point": {
                  "name": name or self._rand_pap_name(),
                  "lacp": lacp,
                  "hash_mode": hash_mode,
                  "interfaces": interfaces}}

        resp, body = self._create_request(PG_PAP, pap, params=params)
        return resp, body

    def show_pap(self, uuid, params=None):
        """Gets a specific physical attachment points.
        :param uuid: Unique identifier of the physical attachment point
                     in UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: Serialized physical attachment point as a dictionary.
        """
        return self._show_request(PG_PAP, uuid, params=params)

    def list_paps(self, params=None):
        """Gets a list of all existing physical attachment points.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: Serialized physical attachment points as a list.
        """
        return self._list_request(PG_PAP, params=params)

    def delete_pap(self, uuid, params=None):
        """Deletes a physical attachment point having the specified UUID.
        :param uuid: The unique identifier of the physical attachment point.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._delete_request(PG_PAP, uuid, params=params)
        return resp, body

    def update_pap(self, uuid, name=None, lacp=False, hash_mode='L2',
                   add_interfaces=[], remove_interfaces=[], params=None):
        """Create a PAP with the specified parameters.

        :param uuid: The unique identifier of the Physical Attachment Point.
        :param name: The name of the Physical Attachment Point.
            Default: Random Value
        :param lacp: enable/disable LACP on Physical Attachment Point.
            Default: False
        :param hash_mode: hash_mode config for Physical Attachment Point.
            Default: L2
        :param interfaces: List of gateway interfaces.
            Default: None
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the created pap.
        """
        pap = {"physical_attachment_point": {
                  "name": name or self._rand_pap_name(),
                  "lacp": lacp,
                  "hash_mode": hash_mode,
                  "add_interfaces": add_interfaces,
                  "remove_interfaces": remove_interfaces}}

        resp, body = self._update_request(PG_PAP, uuid, pap, params=params)
        return resp, body

    def _rand_pap_name(self, name='', prefix='test_pap'):
        return data_utils.rand_name(name=name, prefix=prefix)



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import httplib
from oslo_log import log as logging
from oslo_serialization import jsonutils
from six.moves.urllib import parse as urllib
from tempest.lib.common import rest_client


LOG = logging.getLogger(__name__)


class ClientBase(rest_client.RestClient):
    """Base Tempest REST client for  PLUMgrid Tempest plugin"""

    uri_prefix = ''

    def serialize(self, object_dict):
        return jsonutils.dumps(object_dict)

    def deserialize(self, object_str):
        return jsonutils.loads(object_str)

    def expected_success(self, expected_code, read_code):
        # the base class method does not check correctly if read_code is not
        # an int. warn about this and cast to int to avoid silent errors.
        if not isinstance(read_code, int):
            message = ("expected_success(%(expected_code)r, %(read_code)r) "
                       "received not-int read_code %(read_code)r" %
                       {'expected_code': expected_code,
                        'read_code': read_code})
            LOG.warn(message)
        return super(ClientBase, self).expected_success(
            expected_code=expected_code, read_code=int(read_code),
        )

    def get_uri(self, resource_name, uuid=None, params=None):
        """Get URI for a specific resource or object.
        :param resource_name: The name of the REST resource, e.g.,
                              'physical-attachment-points'.
        :param uuid: The unique identifier of an object in UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :returns: Relative URI for the resource or object.
        """
        uri_pattern = '{pref}/{res}{uuid}{params}'

        uuid = '/%s' % uuid if uuid else ''
        params = '?%s' % urllib.urlencode(params) if params else ''

        return uri_pattern.format(pref=self.uri_prefix,
                                  res=resource_name,
                                  uuid=uuid,
                                  params=params)

    def _create_request(self, resource, object_dict, params=None,
                        headers=None, extra_headers=False):
        """Create an object of the specified type.
        :param resource: The name of the REST resource, e.g.,
                         'physical-attachment-points'.
        :param object_dict: A Python dict that represents an object of the
                            specified type.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :param extra_headers (bool): Boolean value than indicates if the
                                     headers returned by the get_headers()
                                     method are to be used but additional
                                     headers are needed in the request
                                     pass them in as a dict.
        :returns: A tuple with the server response and the deserialized created
                 object.
        """
        body = self.serialize(object_dict)
        uri = self.get_uri(resource, params=params)

        resp, body = self.post(uri, body=body, headers=headers,
                               extra_headers=extra_headers)
        self.expected_success(httplib.CREATED, resp.status)

        return resp, self.deserialize(body)

    def _show_request(self, resource, uuid, params=None):
        """Gets a specific object of the specified type.
        :param resource: The name of the REST resource, e.g.,
                         'physical-attachment-points'.
        :param uuid: Unique identifier of the object in UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :returns: Serialized object as a dictionary.
        """
        uri = self.get_uri(resource, uuid=uuid, params=params)

        resp, body = self.get(uri)

        self.expected_success(httplib.OK, resp.status)

        return resp, self.deserialize(body)

    def _list_request(self, resource, params=None):
        """Gets a list of objects.
        :param resource: The name of the REST resource, e.g.,
                         'physical-attachment-points'.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :returns: Serialized object as a dictionary.
        """
        uri = self.get_uri(resource, params=params)

        resp, body = self.get(uri)

        self.expected_success(httplib.OK, resp.status)

        return resp, self.deserialize(body)

    def _update_request(self, resource, uuid, object_dict, params=None):
        """Updates the specified object.
        :param resource: The name of the REST resource, e.g.,
                         'physical-attachment-points'
        :param uuid: Unique identifier of the object in UUID format.
        :param object_dict: A Python dict that represents an object of the
                             specified type.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :returns: Serialized object as a dictionary.
        """
        body = self.serialize(object_dict)
        uri = self.get_uri(resource, uuid=uuid, params=params)

        resp, body = self.put(uri, body=body)

        self.expected_success(httplib.OK, resp.status)

        return resp, self.deserialize(body)

    def _delete_request(self, resource, uuid, params=None):
        """Deletes the specified object.
        :param resource: The name of the REST resource, e.g.,
                         'physical-attachment-points'.
        :param uuid: The unique identifier of an object in UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :returns: A tuple with the server response and the response body.
        """
        uri = self.get_uri(resource, uuid=uuid, params=params)

        resp, body = self.delete(uri)
        self.expected_success(httplib.NO_CONTENT, resp.status)
        if resp.status == httplib.ACCEPTED:
            body = self.deserialize(body)

        return resp, body



# Copyright 2016 PLUMgrid Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#    http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import httplib
from oslo_log import log as logging
from tempest import config
from tempest import test

from plumgrid_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


def _process_phys_ifcs():
    ifc_list = []
    ifcs_str = CONF.plumgrid.interfaces
    for ifc in ifcs_str.split(','):
        hostname = ifc.split('+')[0]
        for interface in ifc.split('+')[1:]:
            ifc_list.append({'hostname': hostname,
                             'interface': interface})
    return ifc_list


class TestPhysicalAttachmentPoint(base.BaseTest):
    credentials = ['primary', 'admin']
    interfaces = _process_phys_ifcs()

    @classmethod
    def setup_clients(cls):
        super(TestPhysicalAttachmentPoint, cls).setup_clients()

        cls.client = cls.os.pap_client
        cls.admin_client = cls.os_adm.pap_client

    @test.attr(type='smoke')
    @test.idempotent_id('2fb95a42-482d-45cd-93fd-9e161a709874')
    def test_create_pap(self):
        resp, pap = self.admin_client.create_pap()
        self.addCleanup(self.admin_client.delete_pap,
                        pap["physical_attachment_point"]["id"])
        self.assertEqual(resp.status, httplib.CREATED)

    @test.attr(type='smoke')
    @test.idempotent_id('5d4b2eb6-5ce6-44c3-afb3-b0fb75a15c7c')
    def test_show_pap(self):
        _, pap = self.admin_client.create_pap()
        self.addCleanup(self.admin_client.delete_pap,
                        pap["physical_attachment_point"]["id"])
        _, body = self.admin_client.show_pap(
                       pap['physical_attachment_point']['id'])

        self.assertEqual(pap['physical_attachment_point'],
                         body['physical_attachment_point'])

    @test.attr(type='smoke')
    @test.idempotent_id('3ba5892a-5fb1-49fc-b113-1e3b47712a37')
    def test_update_pap(self):
        resp, pap = self.admin_client.create_pap()
        self.addCleanup(self.admin_client.delete_pap,
                        pap["physical_attachment_point"]["id"])
        resp, updated_pap = self.admin_client.update_pap(
                          pap['physical_attachment_point']['id'],
                          add_interfaces=self.interfaces)
        self.assertEqual(resp.status, httplib.OK)

    @test.attr(type='smoke')
    @test.idempotent_id('3f817924-7ca0-41a3-bf30-ce00079a90d6')
    def test_delete_pap(self):
        _, pap = self.admin_client.create_pap(interfaces=self.interfaces)
        resp, pap = self.admin_client.delete_pap(
                     pap['physical_attachment_point']['id'])
        self.assertEqual(resp.status, httplib.NO_CONTENT)

    @test.attr(type='smoke')
    @test.idempotent_id('fb1e3803-2872-439a-bb4a-e3de50d3ed91')
    def test_list_paps(self):
        _, pap1 = self.admin_client.create_pap(interfaces=self.interfaces)
        _, pap2 = self.admin_client.create_pap()
        self.addCleanup(self.admin_client.delete_pap,
                        pap1["physical_attachment_point"]["id"])
        self.addCleanup(self.admin_client.delete_pap,
                        pap2["physical_attachment_point"]["id"])
        resp, paps_list = self.admin_client.list_paps()
        self.assertEqual(resp.status, httplib.OK)
        self.assertEqual(len(paps_list['physical_attachment_points']), 3)

    @test.attr(type='smoke')
    @test.idempotent_id('2fb95a42-482d-45cd-93fd-9e161a709874')
    def test_show_pap_by_name(self):
        _, pap1 = self.admin_client.create_pap()
        _, pap2 = self.admin_client.create_pap()
        self.addCleanup(self.admin_client.delete_pap,
                        pap1["physical_attachment_point"]["id"])
        self.addCleanup(self.admin_client.delete_pap,
                        pap2["physical_attachment_point"]["id"])
        resp, pap_ret = self.admin_client.list_paps({
                            'name': pap1["physical_attachment_point"]["name"]})
        self.assertEqual(pap_ret["physical_attachment_points"][0],
                         pap1["physical_attachment_point"])



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Policy Service extension unit tests
"""
#from networking_plumgrid.neutron.plugins.common import \
#    exceptions as p_excep
#from networking_plumgrid.neutron.plugins.common import \
#    policy_exceptions as policy_excep
from networking_plumgrid.neutron.plugins.extensions import \
    policyservice as ext_ps
from networking_plumgrid.neutron.tests.unit import \
    test_networking_plumgrid as test_pg

from neutron.api import extensions
from neutron.api.v2 import attributes
#from neutron import context
#from neutron import manager
from neutron.tests.unit.api import test_extensions as test_ext
from oslo_log import log as logging
#import uuid

LOG = logging.getLogger(__name__)


class PolicyServiceExtensionManager(object):

    def get_resources(self):
        return ext_ps.Policyservice.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class PolicyServiceTestCase(test_pg.PLUMgridPluginV2TestCase):
    def setUp(self, plugin=None, ext_mgr=None):
        super(PolicyServiceTestCase, self).setUp()
        ext_mgr = PolicyServiceExtensionManager()
        extensions.PluginAwareExtensionManager._instance = None
        self.ext_api = test_ext.setup_extensions_middleware(ext_mgr)


class TestPolicyService(PolicyServiceTestCase):

    def test_create_policy_service_inport(self):
        pass

    def test_create_policy_service_eport(self):
        pass

    def test_create_policy_service_bidirect_port(self):
        pass

    def test_create_policy_service_inport_name(self):
        pass

    def test_create_policy_service_eport_name(self):
        pass

    def test_create_policy_service_bidirect_port_name(self):
        pass

    def test_create_policy_service_dup_ports_inport(self):
        pass

    def test_create_policy_service_dup_ports_eport(self):
        pass

    def test_create_policy_service_dup_ports_bidirect(self):
        pass

    def test_create_policy_service_dup_ports_across_config(self):
        pass

    def test_create_policy_service_without_leg_mode(self):
        pass

    def test_create_policy_service_inport_owner_not_nova(self):
        pass

    def test_create_policy_service_eport_owner_not_nova(self):
        pass

    def test_create_policy_service_bidirect_port_owner_not_nova(self):
        pass

    def test_create_policy_service_inport_port_not_found(self):
        pass

    def test_create_policy_service_eport_port_not_found(self):
        pass

    def test_create_policy_service_bidirect_port_not_found(self):
        pass

    def test_create_policy_service_port_already_in_use_ps(self):
        pass

    def test_delete_policy_service(self):
        pass

    def test_delete_policy_service_ports_in_use(self):
        pass

    def test_delete_policy_service_does_not_exist(self):
        pass

    def test_update_policy_service_name(self):
        pass

    def test_update_policy_service_inport(self):
        pass

    def test_update_policy_service_eport(self):
        pass

    def test_update_policy_service_bidirect_port(self):
        pass

    def test_update_policy_service_add_dup_inports(self):
        pass

    def test_update_policy_service_add_dup_eports(self):
        pass

    def test_update_policy_service_add_dup_bidirect_ports(self):
        pass

    def test_update_policy_service_remove_dup_inports(self):
        pass

    def test_update_policy_service_remove_dup_eports(self):
        pass

    def test_update_policy_service_remove_dup_bidirect_ports(self):
        pass

    def test_update_policy_service_dup_ports(self):
        pass

    def test_update_policy_service_without_leg_mode(self):
        pass

    def test_update_policy_service_add_inports_owner_not_nova(self):
        pass

    def test_update_policy_service_add_eports_owner_not_nova(self):
        pass

    def test_update_policy_service_add_bidirect_ports_owner_not_nova(self):
        pass

    def test_update_policy_service_remove_inports_owner_not_nova(self):
        pass

    def test_update_policy_service_remove_eports_owner_not_nova(self):
        pass

    def test_update_policy_service_remove_bidirect_ports_owner_not_nova(self):
        pass

    def test_update_policy_service_add_inport_already_in_use(self):
        pass

    def test_update_policy_service_add_eport_already_in_use(self):
        pass

    def test_update_policy_service_add_bport_already_in_use(self):
        pass

    def test_create_get_policy_service(self):
        pass

    def test_create_update_get_policy_service_add_ports(self):
        pass

    def test_create_update_get_policy_service_remove_ports(self):
        pass

    def test_list_policy_services(self):
        pass

    def _make_ps_dict(self, name="test_ps", inports=[], eports=[],
                      bidirect_ports=[]):
        return {"policy_service": {
                    "tenant_id": "test_tenant",
                    "description": "test-ps",
                    "name": name,
                    "ingress_ports": inports,
                    "egress_ports": eports,
                    "bidirectional_ports": bidirect_ports}}

    def _make_ps_update_dict(self, name="updated_ps", add_inports=[],
                             add_eports=[], add_bidirect_ports=[],
                             remove_inports=[], remove_eports=[],
                             remove_bidirect_ports=[]):
        return {"policy_service": {
                     "tenant_id": "test_tenant",
                     "name": name,
                     "add_ingress_ports": add_inports,
                     "add_egress_ports": add_eports,
                     "add_bidirectional_ports": add_bidirect_ports,
                     "remove_ingress_ports": remove_inports,
                     "remove_egress_ports": remove_eports,
                     "remove_bidirectional_ports": remove_bidirect_ports}}

    def _fake_network(self):
        data = {'network': {'name': "fake_network",
                            'admin_state_up': False,
                            'shared': False,
                            'router:external': [],
                            'provider:network_type': None,
                            'provider:segmentation_id': None,
                            'provider:physical_network': None}}
        return data

    def _fake_subnet(self, net_id):
        allocation_pools = [{'start': '10.0.0.2',
                             'end': '10.0.0.254'}]
        return {'subnet': {'name': "fake_subnet",
                           'network_id': net_id,
                           'gateway_ip': '10.0.0.1',
                           'dns_nameservers': ['10.0.0.2'],
                           'host_routes': [],
                           'cidr': '10.0.0.0/24',
                           'allocation_pools': allocation_pools,
                           'enable_dhcp': True,
                           'ip_version': 4}}

    def _fake_port(self, net_id, name="fake_port", security_groups = [],
                   device_owner="compute:nova"):
        return {'port': {'name': name,
                         'network_id': net_id,
                         'mac_address': attributes.ATTR_NOT_SPECIFIED,
                         'fixed_ips': attributes.ATTR_NOT_SPECIFIED,
                         'admin_state_up': True,
                         'device_id': 'fake_device_id',
                         'device_owner': device_owner,
                         'tenant_id': "fake_tenant",
                         'security_group': security_groups}}



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_config import cfg
from tempest import config

plumgrid_group = cfg.OptGroup(name='plumgrid',
                         title='PLUMgrid tempest configuration options')

PLUMgridGroup = [
    cfg.StrOpt('interfaces',
               default='',
               help="List of gateway interfaces"),
]

_opts = [(plumgrid_group, PLUMgridGroup)]


def register_opts(conf):
    for g, o in _opts:
        config.register_opt_group(conf, g, o)


def list_opts():
    """Return a list of oslo.config options available.

    The purpose of this is to allow tools like the Oslo sample config file
    generator to discover the options exposed to users.
    """
    return [(g.name, o) for g, o in _opts]



# Copyright 2016 PLUMgrid Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#    http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import httplib
from oslo_log import log as logging
from tempest import test

from plumgrid_tempest_plugin.tests import base

LOG = logging.getLogger(__name__)


class TestTransitDomains(base.BaseTest):
    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(TestTransitDomains, cls).setup_clients()

        cls.client = cls.os.pap_client
        cls.admin_client = cls.os_adm.td_client

    @test.attr(type='smoke')
    @test.idempotent_id('29a30571-2f4c-4bc6-87c4-6f8052e04f93')
    def test_create_td(self):
        resp, td = self.admin_client.create_td()
        self.addCleanup(self.admin_client.delete_td,
                        td["transit_domain"]["id"])
        self.assertEqual(resp.status, httplib.CREATED)

    @test.attr(type='smoke')
    @test.idempotent_id('9ff01991-a7df-4a1e-b2e9-5bc0f5dc21fb')
    def test_show_td(self):
        _, td = self.admin_client.create_td()
        self.addCleanup(self.admin_client.delete_td,
                        td["transit_domain"]["id"])
        _, body = self.admin_client.show_td(
                       td["transit_domain"]["id"])

        self.assertEqual(td['transit_domain'],
                         body['transit_domain'])

    @test.attr(type='smoke')
    @test.idempotent_id('89b2e4be-1cae-4a88-b6e3-8c100ceebba5')
    def test_update_td(self):
        resp, td = self.admin_client.create_td()
        self.addCleanup(self.admin_client.delete_td,
                        td["transit_domain"]["id"])
        resp, updated_td = self.admin_client.update_td(
                               td['transit_domain']['id'],
                               name='test_td_new')
        self.assertEqual(resp.status, httplib.OK)

    @test.attr(type='smoke')
    @test.idempotent_id('61390bee-9d9a-4728-bcfc-439c112c061e')
    def test_delete_td(self):
        _, td = self.admin_client.create_td()
        resp, td = self.admin_client.delete_td(
                     td['transit_domain']['id'])
        self.assertEqual(resp.status, httplib.NO_CONTENT)

    @test.attr(type='smoke')
    @test.idempotent_id('0f2cc7dc-049b-43fa-a7b0-41c044bcc09f')
    def test_list_tds(self):
        _, td1 = self.admin_client.create_td()
        _, td2 = self.admin_client.create_td()
        self.addCleanup(self.admin_client.delete_td,
                        td1["transit_domain"]["id"])
        self.addCleanup(self.admin_client.delete_td,
                        td2["transit_domain"]["id"])
        resp, tds_list = self.admin_client.list_tds()
        self.assertEqual(resp.status, httplib.OK)
        self.assertEqual(len(tds_list['transit_domains']), 3)
